classdef CMMOP10< PROBLEM
% <multi> <real> <multimodal><constrained>
% Constrained multi-modal multi-objective test function

    methods
        %% Default settings of the problem
        function Setting(obj)
            obj.M = 2;
            obj.D = 2;
            obj.lower    = [1,-1];
            obj.upper    = [3,1];
            obj.encoding = 'real';
        end
        %% Calculate objective values
        function PopObj = CalObj(obj,X)
%             g = 1 + sum(1 - exp(-10*((X(:,obj.M:end).^(obj.D-obj.M)) - 0.5 - (repmat(obj.M:obj.D,size(X,1),1) - 1)/(2*obj.D)).^2),2);
            PopObj(:,1) = abs(X(:,1)-2);
            PopObj(:,2) = 1 - PopObj(:,1) + 2*(X(:,2) - sin(6*pi*PopObj(:,1)+pi)).^2;
        end
        
        %% Calculate constraint violations
        function PopCon = CalCon(obj,X)
            PopObj = obj.CalObj(X);
            l      = sqrt(2)*PopObj(:,2) - sqrt(2)*PopObj(:,1);
            PopCon(:,1) = sum(PopObj,2) - 1.05 - 0.45*sin(0.75*pi*l).^6;
            PopCon(:,2) = 0.85 - sum(PopObj,2) + 0.3*sin(0.75*pi*l).^2;
       end
        
        %% Generate Pareto optimal solutions
        function R = GetOptimum(obj,N)
            R(:,1) = linspace(1,3,N)';
            R(:,2) = sin(6*pi*abs(R(:,1)-2)+pi);
            PopObj = obj.CalObj(R);
            l      = sqrt(2)*PopObj(:,2) - sqrt(2)*PopObj(:,1);
            fes1 = PopObj(:,1) + PopObj(:,2) - 1.05 - 0.45*sin(0.75*pi*l).^6 > 0;
            fes2 = PopObj(:,1) + PopObj(:,2) - 0.85 - 0.3*sin(0.75*pi*l).^2 < 0;
            R(fes1|fes2,:) = [];
        end
        
         %% Generate the image of Pareto front
        function R = GetPF(obj)
            [x,y] = meshgrid(linspace(0,1.3,400),linspace(0,1.3,400));
            z     = nan(size(x));
            fes1  = x + y - 1.05 - 0.45*sin(0.75*pi*(sqrt(2)*y-sqrt(2)*x)).^6 <= 0;
            fes2  = 0.85 - x - y + 0.3*sin(0.75*pi*(sqrt(2)*y-sqrt(2)*x)).^2 <= 0;
            z(fes1 & fes2 & x+y>=1) = 0;
            R = {x,y,z};
        end
        
        %% Display a population in the objective space
%         function DrawObj(obj,Population)
%             PopDec = Population.decs;
%             temp   = PopDec(:,2)<=1;
%             Draw(Population(temp).objs,'o','MarkerSize',6,'Marker','o','Markerfacecolor',[1 .5 .5],'Markeredgecolor',[1 .2 .2],{'\it f\rm_1','\it f\rm_2',[]});
%             Draw(Population(~temp).objs+0.1,'o','MarkerSize',6,'Marker','o','Markerfacecolor',[.5 .5 1],'Markeredgecolor',[.2 .2 1]);
%             x = linspace(0,1,100)';
%             y = 1 - x;
%             fes   = x + y - 1 - 0.5*sin(2*pi*(sqrt(2)*y-sqrt(2)*x)).^8 <= 0;
%             x(~fes) = nan;
%             y(~fes) = nan;
%             L = [x,y];
% %             L  = [0:0.01:1;1-(0:0.01:1)]';
% %             scatter(L(:,1),L(:,2),10,'r','filled');
% %             scatter(L(:,1)+0.1,L(:,2)+0.1,10,'b','filled');
%             Draw(L,'-','LineWidth',1,'Color',[1 .2 .2]);
%             Draw(L+0.1,'-','LineWidth',1,'Color',[.2 .2 1]);
%         end
    end
end