classdef CMMOP3 < PROBLEM
% <multi> <real> <multimodal><constrained>
% Constrained multi-modal multi-objective test function

    methods
        %% Default settings of the problem
        function Setting(obj)
            obj.M = 2;
            obj.D = 2;
            obj.lower    = [1,-1];
            obj.upper    = [3,3];
            obj.encoding = 'real';
        end
        %% Calculate objective values
        function PopObj = CalObj(obj,X)
            temp = X(:,2) <= 1;
            y    = zeros(size(X,1),1);
            y(temp)  = X(temp,2) - sin(6*pi*abs(X(temp,1)-2)+pi);
            y(~temp) = X(~temp,2) - 2 - sin(6*pi*abs(X(~temp,1)-2)+pi);
            PopObj(:,1) = abs(X(:,1)-2); 
            PopObj(:,2) = 1 - sqrt(PopObj(:,1)) + 2*y.^2;
        end
        
        %% Calculate constraint violations
        function PopCon = CalCon(obj,X)
            PopObj = obj.CalObj(X);
            l = sqrt(2)*PopObj(:,2) - sqrt(2)*PopObj(:,1);
            PopCon = sqrt(PopObj(:,1)) + PopObj(:,2) - 1 - 0.5*sin(2*pi*l).^8;
        end
        
        %% Generate Pareto optimal solutions
        function R = GetOptimum(obj,N)
            R(:,1) = linspace(1,3,N/2)';
            R(:,2) = sin(6*pi*abs(R(:,1)-2)+pi);
            R = [R;R(:,1),R(:,2)+2];
            PopObj = obj.CalObj(R);
            l = sqrt(2)*PopObj(:,2) - sqrt(2)*PopObj(:,1);
            c = 1 - sqrt(PopObj(:,1)) - PopObj(:,2) + 0.5*sin(2*pi*l).^8;
            R(c<0,:) = [];
        end
        
         %% Generate the image of Pareto front
        function R = GetPF(obj)
            [x,y] = meshgrid(linspace(0,1.3,400));
            z     = nan(size(x));
            fes   = sqrt(x) + y - 1 - 0.5*sin(2*pi*(sqrt(2)*y-sqrt(2)*x)).^8 <= 0;
            z(fes & sqrt(x) + y>=1) = 0;
            R = {x,y,z};
        end
        
        %% Display a population in the objective space
%         function DrawObj(obj,Population)
%             PopDec = Population.decs;
%             temp   = PopDec(:,2)<=1;
%             Draw(Population(temp).objs,'o','MarkerSize',6,'Marker','o','Markerfacecolor',[1 .5 .5],'Markeredgecolor',[1 .2 .2],{'\it f\rm_1','\it f\rm_2',[]});
%             Draw(Population(~temp).objs+0.1,'o','MarkerSize',6,'Marker','o','Markerfacecolor',[.5 .5 1],'Markeredgecolor',[.2 .2 1]);
%             x = linspace(0,1,100)';
%             y = 1 - sqrt(x);
%             fes   = sqrt(x) + y - 1 - 0.5*sin(2*pi*(sqrt(2)*y-sqrt(2)*x)).^8 <= 0;
%             x(~fes) = nan;
%             y(~fes) = nan;
%             L = [x,y];
% %             L  = [0:0.01:1;1-sqrt((0:0.01:1))]';
%             Draw(L,'-','LineWidth',1,'Color',[1 .2 .2]);
%             Draw(L+0.1,'-','LineWidth',1,'Color',[.2 .2 1]);
%         end
    end
end